import Combinatorics: combinations

@inline function _all_formulas(left::Union{Integer, Operation},
                               right::Union{Integer, Operation};
                               )::Set{Union{Integer, Operation}}
    result = Set{Union{Integer, Operation}}()
    if _isvalid(left) && _isvalid(right)
        candidate_formulas = [Add(left, right),
                              Multiply(left, right),
                              Subtract(left, right),
                              Subtract(right, left),
                              Divide(left, right),
                              Divide(right, left)]
        for f in candidate_formulas
            _isvalid(f) && push!(result, f)
        end
    end
    return result
end

@inline function _all_formulas(subset::AbstractVector{<:Integer})::Set{Union{Integer, Operation}}
    n = length(subset)
    if n > 2
        key = Int.(sort(subset))
        cached_result = _formula_cache_read(key)
        if length(cached_result) > 0
            return cached_result
        else
            result = Set{Union{Integer, Operation}}()
            all_indices = collect(1:n)
            for number_left_indices = 1:ceil(Int, n/2)
                for left_indices in combinations(all_indices, number_left_indices)
                    right_indices = setdiff(all_indices, left_indices)
                    for left_formula in _all_formulas(subset[left_indices])
                        for right_formula in _all_formulas(subset[right_indices])
                            for f in _all_formulas(left_formula, right_formula)
                                push!(result, f)
                            end
                        end
                    end
                end
            end
            _formula_cache_write(key, result)
            return result
        end
    elseif n == 2
        return _all_formulas(subset[1], subset[2])
    elseif n == 1
        return Set{Union{Integer, Operation}}(subset[1])
    else
        return Set{Union{Integer, Operation}}()
    end
end
