using CountdownNumbers
using Test

@testset "CountdownNumbers.jl" begin
    @testset "public.jl" begin
        test_cases = [([1, 2, 3, 4, 5, 6], 1, 0),
                      ([1, 2, 3, 4, 5, 6], 2, 0),
                      ([1, 1, 1, 1, 1, 1], 100, 91),
                      ([25, 50, 75, 100, 3, 6], 952, 0),
                      ([50, 75, 100, 25, 2, 1], 940, 0),
                      ([100, 75, 25, 50, 10, 6], 873, 1),
                      ([100, 75, 5, 6, 8, 7], 561, 0),
                      ([1, 1, 1, 1, 1, 1], 200, 191),
                      ([1, 2, 3, 4], 25, 0),
                      ([25, 50, 9, 5, 6, 10], 712, 1),]
        num_test_cases = length(test_cases)
        for i = 1:num_test_cases
            @info("Running test case $(i) of $(num_test_cases)")
            (xs, target, expected_error) = test_cases[i]
            result_formula = numbers(xs, target)
            result_value = CountdownNumbers._value(result_formula)
            result_string = CountdownNumbers._as_string(result_formula)
            result_string_with_parens = CountdownNumbers._as_string_with_parens(result_formula)
            @test eval(Meta.parse(result_string)) == result_value
            @test eval(Meta.parse(result_string_with_parens)) == result_value
            @test abs(target - result_value) == expected_error
        end
    end
    
    @testset "generate_formulas.jl" begin
        @test CountdownNumbers._all_formulas(Int[]) == Set{Union{Integer, CountdownNumbers.Operation}}()
    end
    @testset "show.jl" begin
        @testset "show" begin
            test_cases = [([1, 1], 2, "CountdownNumbers.Operation: 1 + 1 = 2")]
            num_test_cases = length(test_cases)
            for i = 1:num_test_cases
                (xs, target, expected_show_output) = test_cases[i]
                result_formula = numbers(xs, target)
                buffer = IOBuffer()
                print(buffer, result_formula)
                actual_show_output = String(take!(buffer))
                @test actual_show_output == expected_show_output
            end
        end
        @testset "_as_string" begin
            test_cases = [([1, 1], 2, "1 + 1")]
            num_test_cases = length(test_cases)
            for i = 1:num_test_cases
                (xs, target, expected_string) = test_cases[i]
                result_formula = numbers(xs, target)
                @test CountdownNumbers._as_string(result_formula) == expected_string
            end
        end
        @testset "_as_string_with_parens" begin
            test_cases = [([1, 1], 2, "(1 + 1)")]
            num_test_cases = length(test_cases)
            for i = 1:num_test_cases
                (xs, target, expected_string) = test_cases[i]
                result_formula = numbers(xs, target)
                @test CountdownNumbers._as_string_with_parens(result_formula) == expected_string
            end
        end
    end
    @testset "type_constructors.jl" begin
        x = CountdownNumbers.Add(-100, 0, false, -100)
        y = CountdownNumbers.Add(1, 0, false, 1)
        @test CountdownNumbers.Add(x, y) == CountdownNumbers.Add(x, y, false, -99)
        @test CountdownNumbers.Multiply(x, y) == CountdownNumbers.Multiply(x, y, false, -100)
        @test CountdownNumbers.Subtract(x, y) == CountdownNumbers.Subtract(x, y, false, -101)
        @test CountdownNumbers.Divide(x, y) == CountdownNumbers.Divide(x, y, false, -100)
    end
end
